; *********************************************************
; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
;
; Aholab - TSR  1998  http://bips.bi.ehu.es
; Electronics and Telecomunications Dept. - ETSII/IT-Bilbao
; University of the Basque Country
;
; Source name.................. g38kruh.asm
; Package name................. G38kRUH
; Language..................... Motorola 56k assembler
; Status....................... alpha (still uncomplete!!!!!)
; Hard/OS dependencies......... EMV 56002, HAM interface
; Conditional code............. -
;
; Coding: <modem@bips.bi.ehu.es>
;   Itziar Martinez
;   Ricardo Gandara
;   Borja Etxebarria
; Based on freely available sources by Alef Null (see below)
;
;
; Version  yy/mm/dd  Author    Comments
; -------  --------  --------  --------
; 0.1.3    98/01/13  Borja     independent speed selection for TX and RX
; 0.1.2    98/01/08  Borja     Added support for 9600*n; n=1,2,3,4
; 0.1.1    98/01/08  Itziar    KC7WW's interface
; 0.1.0    98/12/30  itziar    initial coding
; ========================= Info ==========================
; G38kRUH -- 38400 bps modem based on the G3RUH 9600 bps:
; Uses 38400 baud two level PAM with fixed rate sampling.
; No adaptive equalizer (yet).
;
;
; Left audio channel is used.
; B port assignments for LEDs/radio interface:
;   bit 0 for PTT
;   bit 2 for DCD (carrier detect)
; Next two bits are (temporarily) used for system load debugging:
;   bit 1: normally off. Will be on during some time if system is
;          overloaded (too much computations for real time)
;   bit 3: on when processing, off when iddel
;
;
;
; Based on original sources by Alef Null Group, modified by N1OWU.
; Some of the information headers from the initial 9600 bps sources:
;
;
; PAM demodulation is based on some ideas presented in book
;      Lee, E., A., Messerschmitt, D., G.:
;      "Digital Communication",
;      Kluwer, 1988
;
; Symbol synchronization is base on the article
;      Mueller, K., H., Mller, M.:
;      "Timing Recovery in Digital Synchronous Data
;      Receivers",
;      IEEE Trans. on Comm., Vol. COM-23, No. 5, May 1976
;
; Copyright (C) 1993-1996 by Alef Null. All rights reserved.
; Author(s): Jarkko Vuori, OH2LNS
;
; This module uses registers as follows:
;  r1 - pointer to the output filter
;  r2 - sample buffer pointer for the demodulator
;  r4 - pointer to the bits for output
;  r5 - sample buffer pointer for the modulator/demodulator
;  r6 - input filter pointer

; =========================================================
;
; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; *********************************************************


        page    132,68,1,1
        opt     rc
	opt	mu
        title   '38kbps modem'

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; User controllable parameters

; For the EVM56002 board:
; TxAtten: a 0dB TxAtten will produce 2.2Vpp // 0.8Vrms aprox. in the
; line out output, or 3.2Vpp // 1.1Vrms in the headphone output.
; RxGain: a 2.2Vpp // 0.8Vrms input signal with a 6dB RxGain will
; use 60% of the A/D conversion dinamic range aprox, (60% is the
; desired value).

kiss    set     1      ; give KISS data (1)/debug data (0) for SPY
iDuplex set     1      ; full-duplex (1)/half-duplex (0) at start-up
RxGain  set     6.0    ; Codec input gain in dB, set to match radio (see paragraph above).
TxAtten set     0      ; Codec output attenuation in dB, set to match radio (see paragraph above).
TxContCarrier  set 0   ; continuous audio output (1) or muted when not xmitting (0)


; decision rate (uncomment only one for tx and one for rx!)
;fd_tx      equ      9600.0
;fd_tx      equ     19200.0
;fd_tx      equ     28800.0
fd_tx      equ     38400.0

;fd_rx      equ      9600.0
;fd_rx      equ     19200.0
;fd_rx      equ     28800.0
fd_rx      equ     38400.0


; End of user controllable parameters
; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

	nolist
	include 'leonid'
	list


; Program parameters

sbuflen  equ     256                                 ; codec sample buffer lenght
fs	equ	48000.0 			    ; sampling frequency
D	equ	5				    ; sample wait

OLevel  set     0.6                                 ; output filter level

; AGC

refLev	equ	0.6				    ; reference level for AGC
agcN	equ	15				    ; agc analyse block lenght
agcGain equ	100.0				    ; agc integrator gain


; symbol synchro

rwfc0   equ     0.99                                ; random walk filter coefficient (carrier off)
rwfc1   equ     0.3                                 ; random walk filter coefficient (carrier on)

; DCD

DCDFil	equ	0.02				    ; decision error IIR LPF coefficient
DCDN    equ     @cvi(0.3*fd_rx+.5)                  ; delay (in s) after DCD off


; G3RUH specific

poly	equ	$10800				    ; G3RUH,K9NG scrambler polynomial (x^17 + x^12 + 1)


; flags

xmit	equ	0				    ; xmit on/off
car	equ	1				    ; carrier on/off


never   set     0

;nueva macro para la espera
; controla tambien el buffer overflow por falta de ciclos de reloj
wblk macro  reg,sbuflen,blklen
            bclr        #3,x:$ffe4    
            clr a
            move        a,x:>pp
            move        x:>pe,a0
            clr b
            cmp a,b
            jeq   _prim
            dec a
            move        a0,x:>pe
            jmp _loop
_prim       bclr        #1,x:$ffe4
_loop       wait                                        ; wait for a new sample to be received
	    clr	a
	    move	x:>pp,a0
	    inc a     	
	    move	a0,x:>pp
	    move		r7,a
	    move		reg,x0
	    sub     x0,a	#>blklen*4+2,x0
	    jmi     <_wrap
	    cmp     x0,a				; wp - rp > threshold
	    jlo     <_loop
	    jmp     <_ok
_wrap       move                #>sbuflen*4,x1
	    add     x1,a				; buffer wraparound, rp - wp + lenght > threshold
	    cmp     x0,a
	    jlo     <_loop
_ok         clr a
            clr     b
            move        #0001,a0
            move        x:>pp,b0
            cmp   b,a
            jcs  _ok_2
            bset    #1,x:$ffe4
            move    #9600,a
            move    a,x:>pe
_ok_2       bset        #3,x:$ffe4

	endm



; macro for the PTT output handling
pttled  macro   mode
        b\mode  #0,x:$ffe4
	endm


; macro for DCD output handling
carled	macro	mode
        b\mode  #2,x:$ffe4
        endm


; first order high-pass IIR filter
; input in a, output in a
; frq is -3dB point (fc/fs)

hpass	macro	acc,frq,store
c1	set	-1.0/@cos(6.28319*frq)*(1.0-@sin(6.28319*frq))
	move		    acc,x0
	move		    #>(1.0-c1)/2.0,y0
	mpyr	x0,y0,acc   x:<store,y0
	move		    acc,x:<store
	sub	y0,acc	    x:<store+1,x0
	move		    #>c1,y0
	macr	-x0,y0,acc
	move		    acc,x:<store+1
	endm

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; Initialization

	org	p:user_code
	move		    #buffer+2,r7	    ; codec sample buffer ptr
        move                #sbuflen*4-1,m7

        move                #buffer,r5              ; sample buffer read ptr
        move                #4-1,n5
        move                #sbuflen*4-1,m5
       
        move                #filtx,r4               ; ptr output bits
	move		    #oftaps-1,m4

        move                #ocoeffs,r1             ; ptr tx filter
        move                #ofnpol,n1
        move                #oftaps*ofnpol-1,m1

;        move                #icoeffs+3*ifnpol*iftaps,r6   ; Not necesary, computed later! ;ptr 1st rx filter = icoeffs+pol(0)+ifnpol*iftaps*sync(3)
        move                #iftaps*ifbnks*ifnpol-1,m6
        move                #ifnpol,n6

        move                #buffer,r2          ;Pactual
        move                #-4,n2              ;pone tres 0 entre muestras      
        move                #sbuflen*4-1,m2

        clr     a
        move               a0,x:>cntr
        move               a0,x:>pol
        move               a0,x:>old
        move               a0,x:>out

        move               #$000001,a0
        move               a0,x:toca

        move               #$000003,a0
        move               a0,x:sync



	if kiss
	move	#reject,a1			    ; serial interface to KISS mode
	move	#ptt,b1
	opensc
	endif

; fs = 48 kHz, mike input, line output.
        ctrlcd  1,r5,sbuflen,MIC,RxGain,RxGain,LINEO|HEADP,TxAtten,TxAtten
        opencd  fs/1000.0,NOHPF
	
  
; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; Main loop

; wait for D samples
loop    wblk r5,sbuflen,D

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; Modulator

        move    #D,a0       ; number of samples to generate
        move    a0,x:<nsamp
filtro  clr     b           (r5)+                       ;filter the bit
        move                x:(r1)+n1,x0    y:(r4)+,y0
	rep	#oftaps-1
        mac     x0,y0,b     x:(r1)+n1,x0    y:(r4)+,y0
	macr	x0,y0,b
        move                b,y:(r5)+n5

        ; increment filter phase and if its greater than 5 (ofnpol), substract 5
        if      (fd_tx==38400)  ; increment polyphase by 4
        move    (r1)+n1         
        move    (r1)-
        endif
        if      (fd_tx==28800)  ; increment polyphase by 3
        move    (r1)+
        move    (r1)+
        move    (r1)+
        endif
        if      (fd_tx==19200)  ; increment polyphase by 2
        move    (r1)+
        move    (r1)+
        endif
        if      (fd_tx==9600)  ; increment polyphase by 1
        move    (r1)+
        endif

        move    #ofnpol+ocoeffs,b
        move    r1,a
        cmp     b,a              ; check if polyphase >= 5
        jcs     mout             ; if not, keep on the same bit
        move    (r1)-n1          ; if yes, decrement polyphase by 5...
        move    (r4)-            ; ...and point to next bit to be sent

; Get next bit to be sent
	if kiss
	getbit					    ; fetch a new one
        jne     <_gb1
        endif

	andi	#$fe,ccr			    ; send zero if no data to be sent
_gb1	rol	a	    y:<prvxsym,x0	    ; NRZ-S coding
	not	a
	eor	x0,a
	ror	a	    a1,y:<prvxsym
        clr     a                                   ; scrambler, databit in C flag, result in a (-1 or 1)
	rol	a	    x:<srem,x0
	eor	x0,a	    #-OLevel,x1
	lsr	a	    #>poly,x0
	jcc	<_gb2
	eor	x0,a
_gb2	move		    a1,x:<srem
	move		    #OLevel,a
	tcc	x1,a

	if !TxContCarrier  		  ; N1OWU
          jset  #xmit,x:<flag,out1       ; if xmit off, then mute output
          clr   a
	endif

out1    move                a,y:(r4)        ; put bit to output filter

; check if we must generate another sample
mout    clr     a                    ;nsamp--
        move    x:<nsamp,a0
        dec     a
        move    a0,x:<nsamp
        jne     filtro               ;if nsamp>0 jump for another sample


; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; Demodulator

        move    x:<cntr,b
        move    #ifnpol*D,a    ;cntr=cntr+ samples(D)*polyphases_per_sample
        add     a,b
        move    b1,x:<cntr

bucle   move    #0005,a        ;while cntr>=5 go into the demodulator
        move    x:<cntr,b
        cmp     a,b
        jmi     loop
        move    x:<out,a1       ;old=out
        move    a1,x:<old

        jsr     <ptrup          ;update filter pointer


;filter the received sample

        move    r2,x:<guarda          ;preserve the value of r2
        clr     a
        move                x:(r2)+n2,x0        y:(r6)+n6,y0
	rep	#iftaps-1
        mac     x0,y0,a     x:(r2)+n2,x0        y:(r6)+n6,y0
        macr    x0,y0,a     
        move    x:<guarda,r2


; high-pass filter the signal (to reject CS4215 offset)

        hpass   a,5.0/(2.0*fd_rx),hpf                  ; fc = 5 Hz


        
; AGC

	move	a,x0
	move		    y:<agc,y0
	mpy	x0,y0,a
	rep	#8
	asl	a
        move    a,x:<out                        ;save the value in out


; update the parameters of the demodulator

        clr     b    x:<cntr,a  ;cntr=cntr-5
        move    #5,b1
        sub     b,a
        move    a1,x:<cntr
        move    x:<pol,a        ;pol=pol+5
        add     b,a
        move    #ifnpol,b1
doaga   cmp     b,a
        jlo     aqui            ; while pol>=ifnpol
        move    (r2)-n2         ;shift to next input sample
        sub     b,a             ;pol=pol-ifnpol
        jmp     doaga
aqui    move    a1,x:<pol
        move    x:<toca,b1      ;toca=not toca
        not     b1
        move    b1,x:<toca

        jclr    #0,x:<toca,bucle   ; if not toca, jump to the beginning


; make decision (for symbol synchro)

        move    x:<out,a
	tst	a	    #refLev,x0
	tpl	x0,a
	move		    #-refLev,x0
	tmi	x0,a
	move		    y:<d1,x0		    ; update decisions
	move		    x0,y:<d3
	move		    a,y:<d1


; AGC control

        move                x:<out,a
	abs	a
	move		    y:<agcmax,x0
	cmp	x0,a	    #>1,x1
	tlo	x0,a
	move		    a,y:<agcmax
	move		    y:<agcn,a		    ; if one block searched
	sub	x1,a	    #>agcN,x1
	move		    a,y:<agcn
	jne	<_agc
	move		    x1,y:<agcn		    ; calculate error and filter it
	clr	b	    y:<agcmax,a
	move		    b,y:<agcmax
	move		    #refLev+refLev/3,b
	sub	a,b	    #>@pow(2,-5)*agcGain/(fs/agcN),x1  ; rectangular integration
	move	b,x0
	move		    y:<agc,a
	macr	x0,x1,a
	abs	a
	move		    a,y:<agc
_agc


; Symbol synchro

        move                y:<d1,a                 ; z = (d3 - d1) * old
	move		    y:<d3,x0
        sub     x0,a        x:<old,x0
	move	a,x1
	mpyr	x0,x1,a     y:<rwf,x0
	move	a,x1
	move		    y:<rwfilt,b 	    ; filter the zero crossing
	macr	x0,x1,b     #0.3,x0
	cmpm	x0,b				    ; check if limits reached
	jlt	<_sync2
	tst	b
	jpl	_sync1
	jsr	<retard
	clr	b
	jmp	_sync2
_sync1	jsr	<advance
	clr	b
_sync2	move		    b,y:<rwfilt
_sync3


; unscramble symbol, result in C

	move		    y:<d1,a		    ; databit to C
	asl	a
	move		    x:<usrem,b1
	jcc	<unscram
	move		    #>(poly<<1)|1,x0
	eor	x0,b
unscram lsr	b
	move		    b1,x:<usrem


; make symbol decision (with NRZ-S decoding), databit in C, result in C

	rol	b	    y:<prvrsym,x0
	eor	x0,b	    b1,y:<prvrsym
	not	b
	lsr	b


; forward to the HDLC handler

	if kiss
	putbit
	endif


; calculate eye opening

	move		    y:<d1,b
        move                x:<out,x0
	sub	x0,b
	abs	b	    #DCDFil,x1


; filter it (with first order IIR filter)

	move	b,x0
	mpy	x0,x1,b     #(1.0-DCDFil),x0
	move		    y:<eyefilt,x1
	macr	x0,x1,b
	move		    b,y:<eyefilt


; and make decision if carrier detected

	jset	#car,x:<flag,_caron
	move		    #0.16+0.03,x0	    ; check if carrier appeared
	cmp	x0,b	    #>DCDN,x0
	jgt	_car2
	move		    x0,y:<DCDn
	move		    #rwfc1,x0
	move		    x0,y:<rwf
	carled	set
	bset	#car,x:<flag
	caron
	jmp	<_car2
_caron	move		    #0.16-0.03,x0	    ; check if carrier disappeared
	cmp	x0,b	    y:<DCDn,b
	jlt	_car2
	move		    #>1,x0		    ; no carrier after DCDN symbols
	sub	x0,b	    #rwfc0,x0
	move		    b,y:<DCDn
	jne	_car2
	move		    x0,y:<rwf
	carled	clr
	bclr	#car,x:<flag
	caroff
_car2

	if !kiss
	move		    y:<eyefilt,a
	spy
	endif


; all this again, sigh!
        jmp     <bucle


; KISS parameter handling
reject  rts

; transmitter PTT control
ptt     jcc     _pttoff
	pttled	set
	bset	#xmit,x:<flag
	rts
_pttoff pttled	clr
	bclr	#xmit,x:<flag
	rts

; ----------------------------------
; Subroutine: advance sampling (used by sync algorithm)

advance clr     a                       ;sync++
        move    x:<sync,a0
        inc     a
        move    a0,x:<sync
        jclr    #3,x:<sync,ptrup        ;if sync>7...
        clr     b
        move    b0,x:<sync              ;...sync=0
        move    x:<cntr,a               ;cntr=cntr+2
        move    #2,b1
        add     b,a
        move    a1,x:<cntr
        move    x:<pol,a                ;pol=pol-2
        sub     b,a                     
        jhs     act                     ;if pol<0...;
        move    #ifnpol,b1
        add     b,a     (r2)+n2         ;...pol=pol+ifnpol and Pactual--
act     move    a1,x:<pol
        jmp     ptrup

; ----------------------------------
; Subroutine: delay sampling (used by sync algorithm)
retard  clr     a
        move    x:<sync,a0           ;sync--
        dec     a
        move    a0,x:<sync
        jhs     ptrup                ;if sync<0 (-1)...
        move    #7,b0                ;...sync=7
        move    b0,x:<sync               
        clr     b     x:<cntr,a
        move    #2,b1                ;cntr=cntr-2
        sub     b,a
        move    a1,x:<cntr
        move    x:<pol,a             ;pol=pol+2
        add     b,a
        move    #ifnpol,b1
        cmp     b,a
        jlo     adv1                 ;if pol>=ifnpol...
        sub     b,a                  ;...pol=pol-ifnpol
        move    (r2)-n2              ;...and Pactual ++
adv1    move    a1,x:<pol
        jmp     ptrup

; ----------------------------------
; Subroutine: update the pointer to the rx filter
;   ptr = icoeffs+pol+ifnpol*iftaps*sync

ptrup   move    #ifnpol*iftaps,x0
        move    x:<sync,y0
        mpy     x0,y0,a
        move    x:<pol,b
        add     b,a
        move    #icoeffs,b
	add	b,a
        move    a1,r6
        rts

; ----------------------------------

	if !kiss
	spyhere RealSpy 			    ; include spy code if necessary
	endif

        if iDuplex
	org	p:kiss_pars+4
	dc	-1				    ; full-duplex at start-up
	endif

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
; DATA segments

	org	x:user_data
n       dc      8
hpf     ds      2     ; data storage for the input DC-cancellation IIR filter
srem	ds	1
usrem	ds	1
flag	dc	0

; demodulator parameters
nsamp   ds      1
cntr    ds      1
pol     ds      1
toca    ds      1
sync    ds      1
old     ds      1
out     ds      1
guarda  ds      1
pp	ds	1
pe      ds      1

; The 9600, 19200 and 28800 filter is exactly the same one. So
; That's why we always use t38k03.asm for those speeds below.
; The 38kbps is different: uses roll-off=0.2, as 0.3 is outside
; the available bandwidth
        org     x:$0040
        if      (fd_tx==38400)
        include 't38k02.asm'
        endif
        if      (fd_tx==28800)
        include 't38k03.asm'
        endif
        if      (fd_tx==19200)
        include 't38k03.asm'
        endif
        if      (fd_tx==9600)
        include 't38k03.asm'
        endif
        if      (ofnpol!=5)   ; if ofnpol is not 5, we force an error
        ofnpol must be 5!!!!
        endif

        org     x:$0800
buffer  dsm     sbuflen*4

	org	y:user_data
d1      ds      1
d3	ds	1
rwfilt	dc	0
eyefilt dc	0
DCDn	dc	DCDN
rwf	dc	rwfc0
prvrsym ds	1				    ; previous received symbol
prvxsym ds	1				    ; previous xmitted symbol
agc     dc      @pow(2,-7)      
agcn	dc	agcN				    ; agc block counter
agcmax	dc	0
filtx   dsm     oftaps

        org     y:$0C00
        if      (fd_rx==38400)
        include 'r38k02.asm'
        endif
        if      (fd_rx==28800)
        include 'r28k03.asm'
        endif
        if      (fd_rx==19200)
        include 'r19k03.asm'
        endif
        if      (fd_rx==9600)
        include 'r9k03.asm'
        endif

        org     y:$0800
        dsm     sbuflen*4
	end

; /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
;EOF

