function c38K4dem (beta,fich_name,bitrate,interp,phaselen)

% C38K4DEM (beta,fich_name,bitrate,interp,phaselen)
% beta = raised cosine roll-off factor
% fich_name = output file name (ASM56002 formatted)
% 	      Example 'r38k02.asm'
% bitrate = modem bitrate
% interp = interpolation factor (in order to getnewsrate=brate*10)
% phaselen = n. puntos de cada polifase. (== n. muestras a filtrar)



 
% ********************************************************* 
% /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/ 
% 
% Aholab - TSR  (c) 1998  http://bips.bi.ehu.es
% Electronics and Telecomunications Dept. - ETSII/IT-Bilbao 
% University of the Basque Country 
% 
% Source name.................. c38K4dem.m
% Package name................. G38kRUH 
% Language..................... Matlab 
% Status....................... alpha(+epsilon) 
% Hard/OS dependencies......... - 
% Conditional code............. - 
% 
% Coding: <modem@bips.bi.ehu.es> 
%   Ricardo Gandara
% 
% 
% Version  yy/mm/dd  Author    Comments 
% -------  --------  --------  -------- 
% 0.1.2    99/01/13  borja     gain adjust
% 0.1.1    99/01/10  richie    de-interlacing & multi-speed
% 0.1.0    98/12/15  richie    initial coding 
% 
% ========================= Info ========================== 
% ========================================================= 
% 
% /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/ 
% ********************************************************* 

gain=1;
fases=[-1.125 -0.52 -0.14 0 0.14 0.52 1.125 1.66];

htemp=rxfil(48000,bitrate,beta,interp,phaselen,0);
g=gain/sqrt(sum(htemp.*htemp));  % power normalize
g=g*sqrt(bitrate/48000/interp);  % unit gain in low freqs.
g=g*interp;  % *each* polyphase must be unit gain.

htemp=htemp*g;
stem(htemp);

for i=1:8
   htemp=g*rxfil(48000,bitrate,beta,interp,phaselen,fases(i));
%   plot (htemp);
%   hold on;
   h(i,(1:interp*phaselen))=htemp';
end
%hold off;
fid=fopen (fich_name,'w');

fprintf (fid,'\niftaps equ %d',phaselen);
fprintf (fid,'\nifbnks equ 8');
fprintf (fid,'\nifnpol equ %d',interp);

fprintf (fid,'\n;');
fprintf (fid,'\n;  Demodulation filter: sqrt(raised cosine) with Hanning window');
fprintf (fid,'\n;                       roll-off factor=%1.3f brate=%.0f gain=%1.1f) ',beta,bitrate,gain);
fprintf (fid,'\n;  Access mode : Start of sync_bank=(interp*phaselen)*(number_of_syncbank-1)');
fprintf (fid,'\n;                Start of polyphase=start_sync_phase +(number_of_polyphase_filter)');
fprintf (fid,'\n;                Filtering tap step = ifnpol ');

fprintf (fid,'\n;');

long=interp*phaselen;
resto=long-floor((long/4))*4;


for i=1:8
   fprintf  (fid,'\n; Sync bank number %d of 8',i);
      if (i==1)
         fprintf (fid,'\nicoeffs dc %1.12f,%1.12f,%1.12f,%1.12f',h(i,1),h(i,2),h(i,3),h(i,4));
      else
         fprintf (fid,'\n        dc %1.12f,%1.12f,%1.12f,%1.12f',h(i,1),h(i,2),h(i,3),h(i,4));
      end
      for j=1:(floor(long/4))-2
         fprintf (fid,'\n        dc %1.12f,%1.12f,%1.12f,%1.12f',h(i,4*j+1),h(i,4*j+2),h(i,4*j+3),h(i,4*j+4));
      end
      k=floor(long/4)-1;
      switch (resto)
      case 0,
         fprintf (fid,'\n        dc %1.12f,%1.12f,%1.12f,%1.12f',h(i,4*k+1),h(i,4*k+2),h(i,4*k+3),h(i,4*k+4));
      case 3,
         fprintf (fid,'\n        dc %1.12f,%1.12f,%1.12f',h(i,4*k+1),h(i,4*k+2),h(i,4*k+3));
      case 2,
         fprintf (fid,'\n        dc %1.12f,%1.12f',h(i,4*k+1),h(i,4*k+2));
      case 1,
         fprintf (fid,'\n        dc %1.12f',h(i,4*k+1));
      end
   fprintf (fid,'\n');
   fprintf (fid,'\n');

end
fclose (fid);



