function f=rxfil(srate, brate, beta, interp, phaselen, delay )
%RXFIL Filtro receptor modem G3RUH
% f=rxfil(srate, brate, beta, interp, phaselen, delay )
% srate = frecuencia de muestreo del codec (muestras/segundo)
% brate = bitrate del modem (bps)
% beta = factor de roll-off (0-1)
% interp = orden del interpolador para obtener newsrate=brate*10
% phaselen = n. puntos de cada polifase. (== n. muestras a filtrar)
% delay = retardo (en muestras interpoladas (x10)) del filtro.


% *********************************************************
% /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
%
% Aholab - TSR  (c) 1998  http://bips.bi.ehu.es
% Electronics and Telecomunications Dept. - ETSII/IT-Bilbao
% University of the Basque Country
%
% Source name.................. rxfil.m
% Package name................. G38kRUH
% Language..................... Matlab
% Status....................... alpha
% Hard/OS dependencies......... -
% Conditional code............. -
%
% Coding: <modem@bips.bi.ehu.es>
%   Borja Etxebarria
%
%
% Version  yy/mm/dd  Author    Comments
% -------  --------  --------  --------
% 0.1.0    98/12/03  borja     initial coding
%
% ========================= Info ==========================
% =========================================================
%
% /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
% *********************************************************


np=4096;  % ptos. fft (para tener buena precision)

modulo=sqcosspec(np,srate,interp,brate,beta); % rellena modulo
fase=linspace(pi*delay,-pi*delay,np+1);  % genera (des)fase lineal
fase=fftshift(fase);
fase=fase(1:np);
[re,im]=pol2cart(fase',modulo);  % polares a cartesianas
spec=re+i*im;
xf=fftshift(real(ifft(spec)));
winlen=interp*phaselen;
from=floor(np/2+1-(winlen-1)/2);
x=xf(from:from+winlen-1);
delta=mod(phaselen,2);
win=hann(winlen+1+delta,delay);
win=win(1+delta:winlen+delta);
x=x.*win';   % enventanado con hanning
f=x;

% *********************************
function y = sqcosspec(FNP,SRATE,INTERP,BRATE,BETA)
	i=0:FNP-1;
	C=zeros(FNP,1);
	for j=1:FNP/2,
                C(j)=sqrt(rcosfunc(linterpol(j,0,FNP,0,SRATE*INTERP),BRATE,BETA));
%                C(j)=(rcosfunc(linterpol(j,0,FNP,0,SRATE*INTERP),BRATE,BETA));
		C(FNP-j+1)=C(j);
	end
	y=C;

% *********************************
function y=linterpol(x,x0,x1,y0,y1)
	y=(x-x0)/(x1-x0)*(y1-y0)+y0;


% *********************************
function y = rcosfunc(f,fs,b)
	af=abs(f);
	ts=1/fs;
	ts2=2*ts;
	if (af<(1-b)/ts2) y=sqrt(ts);
        else
                if (af>(1+b)/ts2) y=0;
                else
                        d=cos(pi*ts/2/b*(af-(1-b)/ts2));
                        y=sqrt(ts)*d*d;
                end
        end

% *********************************

function y = hann(N,delay)
        x=linspace(-N/2,N/2,N);
        y=ihann(x-delay,N);

% *********************************

function y = ihann(x,N)
  y=(0.5+0.5*cos(x/(N/2)*pi)).*(abs(x)<(N/2));

% *********************************


